// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import edu.wpi.first.wpilibj.shuffleboard.BuiltInWidgets;
import edu.wpi.first.wpilibj.shuffleboard.Shuffleboard;
import edu.wpi.first.wpilibj.shuffleboard.ShuffleboardTab;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.Commands;
import edu.wpi.first.wpilibj2.command.button.CommandXboxController;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine.Direction;
import frc.robot.constants.OIConstants;
import frc.robot.constants.ElevatorConstants.ElevatorPositions;
import frc.robot.constants.OIConstants.RobotOperatingMode;
import frc.robot.subsystems.Elevator;
import frc.robot.subsystems.ElevatorSysID;

public class RobotContainer {
  private Elevator elevator;

  private CommandXboxController driver;

  private RobotOperatingMode operatingMode;

  public RobotContainer() {
    operatingMode = RobotOperatingMode.kNormal;

    switch(operatingMode) {
      case kNormal:
      case kElevatorSetpointDebug:
      default:
        elevator = new Elevator();
        break;
      case kElevatorSysIDCharacterization:
        elevator = new ElevatorSysID();
        break;
    }

    driver = new CommandXboxController(OIConstants.kDriverUSB);

    switch(operatingMode) {
      case kNormal:
      default:
        configureNormalBindings();
        break;
      case kElevatorSetpointDebug:
        configureElevatorSetpointBindings();
        break;
      case kElevatorSysIDCharacterization:
        configureElevatorSysIDBindings();
        break;
    }

    configureShuffleboard();
  }

  private void configureNormalBindings() {
    elevator.setDefaultCommand(
      elevator.assistedManualControl(driver::getRightY)
    );

    driver.a().onTrue(elevator.goToPosition(ElevatorPositions.kBottom, 3));
    driver.b().onTrue(elevator.goToPosition(ElevatorPositions.kLowMid, 3));
    driver.x().onTrue(elevator.goToPosition(ElevatorPositions.kHighMid, 3));
    driver.y().onTrue(elevator.goToPosition(ElevatorPositions.kTop, 3));
  }

  private void configureElevatorSetpointBindings() {
    elevator.setDefaultCommand(
      elevator.debugSetpoints()
    );
  }

  private void configureElevatorSysIDBindings() {
    driver.a().and(driver.rightBumper()).whileTrue(
      ((ElevatorSysID)elevator).sysIdQuasistatic(Direction.kForward)
    );

    driver.b().and(driver.rightBumper()).whileTrue(
      ((ElevatorSysID)elevator).sysIdQuasistatic(Direction.kReverse)
    );

    driver.a().and(driver.leftBumper()).whileTrue(
      ((ElevatorSysID)elevator).sysIdDynamic(Direction.kForward)
    );

    driver.b().and(driver.leftBumper()).whileTrue(
      ((ElevatorSysID)elevator).sysIdDynamic(Direction.kReverse)
    );
  }

  private void configureShuffleboard() {
    ShuffleboardTab tab = Shuffleboard.getTab("MyTab");

    tab.addDouble("Elevator Encoder Distance", elevator::getEncoderDistance)
      .withPosition(0, 0)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Encoder Rate", elevator::getEncoderRate)
      .withPosition(2, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Bottom Setpoint", ElevatorPositions.kBottom::getPositionValue)
      .withPosition(0, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Low-Mid Setpoint", ElevatorPositions.kLowMid::getPositionValue)
      .withPosition(2, 1)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator High-Mid Setpoint", ElevatorPositions.kHighMid::getPositionValue)
      .withPosition(0, 2)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);

    tab.addDouble("Elevator Top Setpoint", ElevatorPositions.kTop::getPositionValue)
      .withPosition(2, 2)
      .withSize(2, 1)
      .withWidget(BuiltInWidgets.kTextView);
  }

  public Command getAutonomousCommand() {
    return Commands.print("No autonomous command configured");
  }
}
